@@ -28,7 +28,7 @@ require utf8;
 } unless defined &utf8::is_utf8;
 
 $DBI::PurePerl = $ENV{DBI_PUREPERL} || 1;
-$DBI::PurePerl::VERSION = sprintf("2.%06d", q$Revision: 11372 $ =~ /(\d+)/o);
+$DBI::PurePerl::VERSION = sprintf("2.%06d", q$Revision: 14285 $ =~ /(\d+)/o);
 
 $DBI::neat_maxlen ||= 400;
 
@@ -130,6 +130,9 @@ use constant IMA_HIDE_ERR_PARAMVALUES => 0x4000; #/* ParamValues are not relevan
 use constant IMA_IS_FACTORY     => 0x8000; #/* new h ie connect & prepare */
 use constant IMA_CLEAR_CACHED_KIDS    => 0x10000; #/* clear CachedKids before call */
 
+use constant DBIstcf_STRICT           => 0x0001;
+use constant DBIstcf_DISCARD_STRING   => 0x0002;
+
 my %is_flag_attribute = map {$_ =>1 } qw(
 	Active
 	AutoCommit
@@ -140,6 +143,7 @@ my %is_flag_attribute = map {$_ =>1 } qw(
 	TaintIn
 	TaintOut
 	InactiveDestroy
+	AutoInactiveDestroy
 	LongTruncOk
 	MultiThread
 	PrintError
@@ -221,7 +225,13 @@ sub  _install_method {
     return if $method_name eq 'can';
 
     push @pre_call_frag, q{
-	return if $h_inner; # ignore DESTROY for outer handle
+        # ignore DESTROY for outer handle (DESTROY for inner likely to follow soon)
+	return if $h_inner;
+        # handle AutoInactiveDestroy and InactiveDestroy
+        $h->{InactiveDestroy} = 1
+            if $h->{AutoInactiveDestroy} and $$ != $h->{dbi_pp_pid};
+        $h->{Active} = 0
+            if $h->{InactiveDestroy};
 	# copy err/errstr/state up to driver so $DBI::err etc still work
 	if ($h->{err} and my $drh = $h->{Driver}) {
 	    $drh->{$_} = $h->{$_} for ('err','errstr','state');
@@ -524,6 +534,7 @@ sub _setup_handle {
 	$h_inner->{Type}                ||= 'dr';
     }
     $h_inner->{"dbi_pp_call_depth"} = 0;
+    $h_inner->{"dbi_pp_pid"} = $$;
     $h_inner->{ErrCount} = 0;
     $h_inner->{Active} = 1;
 }
@@ -562,6 +573,12 @@ sub _set_trace_file {
         $DBI::tfh_needs_close = 0;
         return 1;
     }
+    if ($file && ref \$file eq 'GLOB') {
+	$DBI::tfh = *{$file}{IO};
+        select((select($DBI::tfh), $| = 1)[0]);
+        $DBI::tfh_needs_close = 0;
+        return 1;
+    }
     $DBI::tfh_needs_close = 1;
     if (!$file || $file eq 'STDERR') {
 	open $DBI::tfh, ">&STDERR" or carp "Can't dup STDERR: $!";
@@ -668,6 +685,42 @@ sub neat {
     return "$quote$v$quote";
 }
 
+sub sql_type_cast {
+    my (undef, $sql_type, $flags) = @_;
+
+    return -1 unless defined $_[0];
+
+    my $cast_ok = 1;
+
+    my $evalret = eval {
+        use warnings FATAL => qw(numeric);
+        if ($sql_type == SQL_INTEGER) {
+            my $dummy = $_[0] + 0;
+            return 1;
+        }
+        elsif ($sql_type == SQL_DOUBLE) {
+            my $dummy = $_[0] + 0.0;
+            return 1;
+        }
+        elsif ($sql_type == SQL_NUMERIC) {
+            my $dummy = $_[0] + 0.0;
+            return 1;
+        }
+        else {
+            return -2;
+        }
+    } or $^W && warn $@; # XXX warnings::warnif("numeric", $@) ?
+
+    return $evalret if defined($evalret) && ($evalret == -2);
+    $cast_ok = 0 unless $evalret;
+
+    # DBIstcf_DISCARD_STRING not supported for PurePerl currently
+
+    return 2 if $cast_ok;
+    return 0 if $flags & DBIstcf_STRICT;
+    return 1;
+}
+
 sub dbi_time {
     return time();
 }
@@ -1068,7 +1121,7 @@ bug reports to Jeff Zucker at <jeff@vpservices.com> with a cc to
 =head1 USAGE
 
 The usage is the same as for standard DBI with the exception
-that you need to set the enviornment variable DBI_PUREPERL if
+that you need to set the environment variable DBI_PUREPERL if
 you want to use the PurePerl version.
 
  DBI_PUREPERL == 0 (the default) Always use compiled DBI, die
@@ -1123,6 +1176,7 @@ functionality:
 
   ActiveKids
   InactiveDestroy
+  AutoInactiveDestroy
   Kids
   Taint
   TaintIn
@@ -1133,7 +1187,7 @@ functionality:
 =head2 Tracing
 
 Trace functionality is more limited and the code to handle tracing is
-only embeded into DBI:PurePerl if the DBI_TRACE environment variable
+only embedded into DBI:PurePerl if the DBI_TRACE environment variable
 is defined.  To enable total tracing you can set the DBI_TRACE
 environment variable as usual.  But to enable individual handle
 tracing using the trace() method you also need to set the DBI_TRACE
